@@ -96,6 +96,11 @@ module NumRu
       #   this argument can be used to specify the dimensions that are
       #   not included in +grid_locs+ and are treated as independent, so
       #   the fitting is made for each of their component.
+      # * +with_offset+ (optional) if true (default), constant offset is derived.
+      #   If false, offset is not treated.
+      # * +wgt_func+ (optional) [nil (defualt) or Proc] If Proc, weighting function
+      #   to conduct the weighted least square fit. It should conform with the
+      #   fitting functions.
       # 
       # Note that the sum of the lengths of +grid_locs+, +ensemble_dims+ and
       # +indep_dims+ must be equal to the rank (# of dims) of +data+.
@@ -215,7 +220,7 @@ module NumRu
       #   number of functions (also unsolvable).
       # 
       def least_square_fit(data, grid_locs, functions, ensemble_dims=nil,
-                           indep_dims=nil, with_offset=true)
+                           indep_dims=nil, with_offset=true, wgt_func = nil)
 
         #< argument check >
 
@@ -269,12 +274,20 @@ module NumRu
           if otherdims.length != ensemble_dims.length + n_indep
             raise ArgumentError, "Overlap in ensemble_dims and indep_dims"
           end
+          sh = data.shape
+          indep_len = 1
+          indep_dims.each{|i| indep_len *= sh[i]}
+          meandims = (0...rank).collect{|d| d} - indep_dims
         end
 
         #< pre-process data >
 
-        d0 = data.mean
-        data = data - d0     # constant offset for numerical stability
+        if with_offset
+          d0 = data.mean
+          data = data - d0     # constant offset for numerical stability
+        else
+          d0 = 0.0
+        end
 
         if data.is_a?(NArrayMiss)
           mask = data.get_mask
@@ -291,33 +304,66 @@ module NumRu
           otherdims.each{|d| f.newdim!(d)}
           f
         }
-
+        if wgt_func
+          wgt = wgt_func[*grid_locs]
+          otherdims.each{|d| wgt.newdim!(d)}
+        else
+          wgt = nil
+        end
+        
         ms = fv.length    # matrix size
 
         if ( (len=data.length) < ms )
           raise "Insufficient data length (#{len}) for the # of funcs+1 (#{ms})"
         end
 
-        mat = NMatrix.float(ms,ms)   #  wil be symmetric
+        if mask && indep_dims
+          mat = (0...indep_len).map{NMatrix.float(ms,ms)}
+        else
+          mat = NMatrix.float(ms,ms)   #  wil be symmetric
+        end
 
         for i in 0...ms
           for j in 0..i
             if mask
-              fvij = NArrayMiss.to_nam( fv[i] * fv[j] * mask, mask )
-              mat[i,j] = (fvij).mean
+              unless wgt
+                fvij = NArrayMiss.to_nam( fv[i] * fv[j] * mask, mask )
+              else
+                fvij = NArrayMiss.to_nam( fv[i] * fv[j] * wgt * mask, mask )
+              end
+              if !indep_dims
+                mat[i,j] = (fvij).mean
+              else
+                x = (fvij).mean(*meandims)
+                (0...indep_len).each{|k| mat[k][i,j] = x[k]}
+              end
             else
-              mat[i,j] = (fv[i] * fv[j]).mean
+              unless wgt
+                mat[i,j] = (fv[i] * fv[j]).mean
+              else
+                mat[i,j] = (fv[i] * fv[j] * wgt).mean
+              end
             end
           end
         end
 
         for i in 0...ms
           for j in i+1...ms
-            mat[i,j] = mat[j,i]      # symmetric
+            if mask && indep_dims
+              for k in 0...indep_len
+                mat[k][i,j] = mat[k][j,i]      # symmetric
+              end
+            else
+              mat[i,j] = mat[j,i]      # symmetric
+            end
           end
         end
         #p "*** mat ***",mat
-        lu = mat.lu
+        if mask && indep_dims
+          lu = (0...indep_len).map{|k| mat[k].lu}
+        else
+          lu = mat.lu
+        end
 
         #< derive the vector, solve, and best fit >
 
@@ -325,7 +371,11 @@ module NumRu
           # derive the vector
           b = NVector.float(ms)
           for i in 0...ms
-            b[i] = (data * fv[i]).mean
+            unless wgt
+              b[i] = (data * fv[i]).mean
+            else
+              b[i] = (data * fv[i] * wgt).mean
+            end
           end
 
           # solve
@@ -355,7 +405,6 @@ module NumRu
           # derive vectors
           idshp = indep_dims.collect{|d| data.shape[d]}
           bs = NArray.float(ms,*idshp)
-          meandims = (0...rank).collect{|d| d} - indep_dims
           for i in 0...ms
             bsi = (data * fv[i]).mean(*meandims)
             if bsi.is_a?(NArrayMiss)
@@ -366,16 +415,18 @@ module NumRu
             end
             bs[i,false] = bsi
           end
-          idlen = 1
-          idshp.each{|l| idlen *= l}
 
           # solve
-          bs = bs.reshape(ms, idlen)
-          c = NArray.float(ms,idlen)
+          bs = bs.reshape(ms, indep_len)
+          c = NArray.float(ms,indep_len)
           b = NVector.float(ms)
-          for id in 0...idlen
+          for id in 0...indep_len
             b[true] = bs[true,id]
-            c[true,id] = lu.solve(b)
+            if mask
+              c[true,id] = lu[id].solve(b)
+            else
+              c[true,id] = lu.solve(b)
+            end
           end
           c[-1,true] += d0
           c = c.reshape(ms, *idshp)
@@ -453,6 +504,11 @@ module NumRu
     #   this argument can be used to specify the dimensions that are
     #   not included in +grid_locs+ and are treated as independent, so
     #   the fitting is made for each of their component.
+    # * +with_offset+ (optional) if true (default), constant offset is derived.
+    #   If false, offset is not treated.
+    # * +wgt_func+ (optional) [nil (defualt) or Proc] If Proc, weighting function
+    #   to conduct the weighted least square fit. It should conform with the
+    #   fitting functions.
     # 
     # === RETURN VALUES
     #   [ c, bf, diff ]
@@ -473,7 +529,8 @@ module NumRu
 
 
 
-    def least_square_fit(functions, ensemble_dims=nil, indep_dims=nil)
+    def least_square_fit(functions, ensemble_dims=nil, indep_dims=nil, with_offset=true,
+                         wgt_func = nil)
 
       #< preparation >
 
@@ -492,7 +549,8 @@ module NumRu
 
       #< fitting >
       c, bf, diff = GAnalysis::Fitting.least_square_fit(data, grid_locs, 
-                                          functions, ensemble_dims, indep_dims)
+                           functions, ensemble_dims, indep_dims,
+                           with_offset, wgt_func)
 
       #< make a GPhys of the best fit >
 
@@ -557,12 +615,17 @@ if $0 == __FILE__
   data = x + x*x*0.1
   c, bf, diff = GAnalysis::Fitting.least_square_fit(data, [x], 
                                               [GAnalysis::Fitting::X])
-  p "data:", data, "c:", c, "bf:", bf
-exit
+  p "*1* data:", data, "c:", c, "bf:", bf, "diff:", diff
 
   c, bf, diff = GAnalysis::Fitting.least_square_fit(data, [x], 
                              [GAnalysis::Fitting::X,GAnalysis::Fitting::XX])
-  p c
+  p "*2* data:", data, "c:", c, "bf:", bf, "diff:", diff
+
+  wgt = Proc.new{|x| 1.0 + x**2}
+  c, bf, diff = GAnalysis::Fitting.least_square_fit(data, [x], 
+                             [GAnalysis::Fitting::X,GAnalysis::Fitting::XX],
+                             nil, nil, true, wgt)
+  p "*3* data:", data, "x:", x, "c:", c, "bf:", bf, "diff:", diff
 
   xx = x.newdim(-1)
   data = xx + 2*yy + 100
