@@ -43,9 +43,9 @@ module NumRu
       val = gphys0.val
       val = val.get_array![val.get_mask!] if NArrayMiss === val
       val = NMath.log10(val) if log_bins
-      hist.increment(val)
+      val.each{|v| hist.increment(v)}
 
-      bounds = hist.range.to_na
+      bounds =  NArray.to_na(hist.range.to_a)
       bounds = 10 ** bounds if log_bins
       center = (bounds[0..-2]+bounds[1..-1])/2
       cell_width = (bounds[1..-1] - bounds[0..-2]) / 2
@@ -57,7 +57,7 @@ module NumRu
       axis.set_cell(center, bounds, name)
       axis.set_pos_to_center
 
-      bin = hist.bin.to_na
+      bin = NArray.to_na(hist.bin.to_a)
       bin /= cell_width if opts["log_bins"]
       bin = VArray.new(bin,
                        {"long_name" => (log_bins ? "number per unit bin width" : "number in bins"), "units"=>"1"},
