@@ -191,26 +191,33 @@ module NumRu
       end
       private :del_ngp
 
-      # Derive geostrophic wind from geopotential hight (spherical but fixed f)
+      # Derive geostrophic wind from geopotential hight (spherical)
       # 
       # ARGUMENTS
       # * z [GPhys] : geopotential height on the pressure (or log-pressure)
       #   coordinate
-      # * f [nil or Numeric of UNumeric] : the constant f value 
-      #   (Coriolis parameter).
-      #   If nil, the value at the north pole is assumed.
-      #   If Numeric, units are assumed to be "s-1".
+      # * f [nil or GPhys or Numeric or UNumeric] : Coriolis parameter
+      #   If nil, local Corioris parameter is used.
+      #   If Numeric or UNumeric, f is a constant as a matter of course
+      #   (if Numeric, units are assumed to be "s-1").
       #
       def z2geostrophic_wind(z, f=nil)
         if f.nil?
-          f = 2 * GAnalysis::Planet.omega
+          f = get_f(z)
         elsif f.is_a?(Numeric)
           f = UNumeric[f,"s-1"]
         end
-        z = z.convert_units("m")
-        gx, gy = GAnalysis::Planet.grad_s( z * (g/f) )
-        u = -gy
-        v = gx
+        #z = z.convert_units("m")
+        gx, gy = GAnalysis::Planet.grad_s( z * g )
+        u = -gy/f
+        v = gx/f
+        if f.respond_to? :val   # supposedly a GPhys
+          if0 = f.val.eq(0).where
+          if if0.length > 0
+            u[true,if0,false] = 0   # replace Inf with 0
+            v[true,if0,false] = 0   # replace Inf with 0
+          end
+        end
         u.name = "u"
         u.long_name = "geostrophic U"
         u.put_att("assumed_f",f.to_s)
@@ -220,6 +227,16 @@ module NumRu
         [u, v]
       end
 
+      # Get the Coriolis parameter
+      def get_f(gp)
+        lam, phi, = GAnalysis::Planet.get_lambda_phi(gp)
+        sin_phi = phi.sin
+        f = 2 * GAnalysis::Planet.omega * sin_phi
+        f.name = "f"
+        f.long_name = "Coriolis parameter"
+        f
+      end
+
       # Adiabatic frontogenesis function over the sphere. --
       # D/Dt(|gradH theta|) or D/Dt(|gradH theta|^2), where gradH express
       # the horizontal component of gradient. 
