@@ -1,5 +1,5 @@
 require "narray"
-require "numru/ganalysis/ganalysis_ext"
+require "numru/ganalysis_ext"
 
 class NArray
   def cum_sum(dim)
