@@ -95,6 +95,82 @@ module NumRu
         rot
       end
 
+      # spherical divergence for entire rectagular regional data
+      def div_s_box(u,v, only_u: false, only_v: false,
+                    xs0: false, xe0: false, ys0: false, ye0: false)
+        lam, phi, xd, yd  = get_lambda_phi(u)
+        lon = u.coord(xd)
+        lat = u.coord(yd)
+        cos_phi = phi.cos
+        nx = lam.length
+        ny = phi.length
+        if nx>2
+          ixends = {0..-1=>(nx-1)}
+        else
+          ixends = true
+        end
+        if ny>2
+          iyends = {0..-1=>(ny-1)}
+        else
+          iyends = true
+        end
+        ub = u[*([true]*xd + [ixends, false])].copy  # ends along lon
+        vb = v[*([true]*yd + [iyends, false])].copy  # ends along lat
+        ub.axis(xd).set_pos(lam[ixends].data)
+        ub.axis(yd).set_pos(phi.data)
+        vb.axis(xd).set_pos(lam.data)
+        vb.axis(yd).set_pos(phi[iyends].data)
+        ubi = ub.integrate(yd)   # trapezoidal integration by default
+        vbi = vb.integrate(xd) * cos_phi[iyends]
+        dlam = lam[-1].val - lam[0].val
+        sin_phib = phi[iyends].sin.val
+        dsin_phi = sin_phib[-1] - sin_phib[0]
+        if xd<yd
+          xdi = xd
+          ydi = yd-1
+        else
+          xdi = xd-1
+          ydi = yd
+        end
+        scalar=false
+        ubi.axis(xdi).set_pos(lon[ixends])  # to lon before the cut is recoreded
+        vbi.axis(ydi).set_pos(lat[iyends])  # to lat before the cut is recoreded
+        if ubi.rank>1
+          xe = ubi[*([true]*xdi+[-1,false])]
+          xs = ubi[*([true]*xdi+[0,false])]
+          ye = vbi[*([true]*ydi+[-1,false])]
+          ys = vbi[*([true]*ydi+[0,false])]
+        else
+          # to defer the result becomes a scalar
+          scalar=true
+          xe = ubi[*([true]*xdi+[-1..-1,false])]
+          xs = ubi[*([true]*xdi+[0..0,false])]
+          ye = vbi[*([true]*ydi+[-1..-1,false])]
+          ys = vbi[*([true]*ydi+[0..0,false])]
+        end
+        xe *= 0.0 if xe0
+        xs *= 0.0 if xs0
+        ye *= 0.0 if ye0
+        ys *= 0.0 if ys0
+        dub = xe - xs
+        dvb = ye - ys
+        if only_u
+          div = dub/(@@R*dlam*dsin_phi.abs) 
+        elsif only_v
+          div = dvb/(@@R*dlam.abs*dsin_phi)
+        else
+          div = dub/(@@R*dlam*dsin_phi.abs) +
+                dvb/(@@R*dlam.abs*dsin_phi)
+              # (dlam*dsin_phi).abs is the area for the unit radius
+        end
+        if scalar
+          div = div[0]
+        end
+        div.long_name = "div(#{u.name},#{v.name})"
+        div.name = "div"
+        div
+      end
+      
       def vor_s(u,v)
         vor = rot_s(u,v)
         vor.long_name = "Relative Vorticity"
@@ -131,7 +207,7 @@ module NumRu
       end
 
       def grad_sy(s)
-        lam, phi, lond, latd  = get_lambda_phi(s)
+        phi, latd  = get_phi(s)
         cos_phi = phi.cos
         ys = s.cderiv(latd,latbc(phi),phi) / @@R
         ys.long_name = "ygrad(#{s.name})"
@@ -196,6 +272,22 @@ module NumRu
         [lam, phi, lond, latd]
       end
 
+      def get_lambda(gp, err_raise=true)
+        lond = find_lon_dim(gp, err_raise)
+        lam = lond && gp.axis(lond).to_gphys.convert_units("rad") # lon in rad
+        lam.units = "" if lond                            # treat as non-dim
+        @@lam = lam
+        [lam, lond]
+      end
+
+      def get_phi(gp, err_raise=true)
+        latd = find_lat_dim(gp, err_raise)
+        phi = latd && gp.axis(latd).to_gphys.convert_units("rad") # lat in rad
+        phi.units = "" if latd                            # treat as non-dim
+        @@phi = phi
+        [phi, latd]
+      end
+
       # Find longitude and latitude coordinates.
       # 
       # ARGUMENTS
@@ -254,6 +346,49 @@ module NumRu
         end
         [lond,latd]
       end
+
+      def find_lon_dim(gp, err_raise=false)
+        lond = nil
+        (0...gp.rank).each do |dim|
+          crd = gp.coord(dim)
+          if /^degrees?_east$/i =~ crd.get_att("units")
+            lond = dim
+            break
+          elsif ( ( /longitude/i =~ crd.long_name || 
+                    /^lon/i =~ crd.name ||
+                    (nm=crd.get_att("standard_name") && /longitude/i =~ nm ) &&
+                    (crd.units =~ Units["degrees_east"]) ) )
+            lond = dim
+            break
+          end
+        end
+        if err_raise
+          raise("Longitude dimension was not found") if !lond
+        end
+        lond
+      end
+
+      def find_lat_dim(gp, err_raise=false)
+        latd = nil
+        (0...gp.rank).each do |dim|
+          crd = gp.coord(dim)
+          if /^degrees?_north$/i =~ crd.get_att("units")
+            latd = dim
+            break
+          elsif ( ( /latitude/i =~ crd.long_name || 
+                    /^lat/i =~ crd.name ||
+                    (nm=crd.get_att("standard_name") && /latitude/i =~ nm ) &&
+                    (crd.units =~ Units["degrees_north"]) ) )
+            latd = dim
+            break
+          end
+        end
+        if err_raise
+          raise("Latitude dimension was not found") if !latd
+        end
+        latd
+      end
+
     end
 
   end
