@@ -213,7 +213,217 @@ module NumRu
         divh
       end
 
+      # Wave activity flux by Takaya and Nakamura (2001) (TN2001)
+      #
+      # This method works by mix-in in QG (Cartesian) or QG_sphere (spherical).
+      # (so the call it either as QG::waf_tn2001(gph, gphbs, ..) or
+      # QG_sphere::waf_tn2001(gph, gphbs, ..) ).
+      #
+      # DEFINITIONS
+      #
+      # * if called through QG: a modified version of Eq (32) of TN2001
+      # * if called through QG_sphere: a modified version of Eq (38) of TN2001
+      # The modification of Eq (32) and (38) is as follows:
+      # * The multiplication by p is omitted. This is because
+      #    it is more convenient to do it afterwords if needed
+      #    (for example, to integrate with p would require not to have
+      #    done this multiplication).
+      # * The C_u M term is not included as in convention.
+      #   (but the multiplication by cos(phi) is done in QG_sphere).
+      #   Note that this term vanishes for stationary waves, because C_u=0.
+      # * The normal vector (n1, n2) in TN2001 can be set flexibly.
+      #   This vector is called (nx, ny) in this method, since n2 is used
+      #   for the buoyancy frequency. In eqs. (32) and (38), (nx, ny)
+      #   is prescribed as (nx, ny) = (U,V)/|U,V| where U is ubs and V is vbs.
+      #   However, this relation is not necessarily valid, since (nx, ny)
+      #   is originally attributed to the QGPV gradient of the basic flow.
+      #   Therefore, in this method, (nx, ny) can be set arbitrary.
+      #   Also, one can set (nx, ny) = (1, 0) or (0, 1) by setting the
+      #   optional argument zonal or merid to true.
+      #
+      # ARGUMENTS
+      # * gph: geopotential height
+      # * gphbs: geopotential height (basic state). Basically, it needs to have
+      #   the same spacial grid as gph, but it can be a meridional section
+      #   (such as zonal mean), if the normal vector is specified by
+      #   (ubs, vbs) or (nx, ny) or zonal or merid.
+      # * nx, ny: normalized vector to designate the direction of wave
+      #   propagation. By default, set to the direction of (ubs, vbs),
+      #   or set by zonal/merid (see below).
+      # * ubs, vbs: Basic state zonal and meridional winds to compute
+      #   the normal vector (nx,ny) as (ubs,vbs)/sqrt(ubs**2 + vbs**2).
+      #   Neglected if nx and ny are specified. By default ubs and vbs
+      #   are derived geostrophically from gphbs
+      # * zonal: derive the flux in the zonal direction assuming (nx,ny)=(1,0)
+      # * merid: derive the flux in the meridional dir assuming (nx,ny)=(0,1)
+      # * n2: Brunt-Vaisala (buoyancy) frequency squared. By default, derived
+      #   from gphbs
+      # * derive_M0: derive M (the wave-activity pseudomomentum of TN20001)
+      #   when the wave is stationary (cp=0). You need to give ubs and vbs
+      #   (or have them derived from gphbs by not using nx, ny, zonal or merid)
+      #   to make this option available.
+      #
+      # RETURN VALUE
+      # * [wx, wy, wz, ret_hash] where [wx, wy, wz] are the WAF vector
+      #   and ret_hash provides extra derived quantities such as the
+      #   perturbation stream function (:pp), pseudomomentum M0 (:M0), etc.
+      #
+      # REFERENCE
+      # * Takaya and Namaura (2001): A formulation of a phase-independent
+      #   wave-activity flux for stationary and migratory quasigeostrophic
+      #   eddies on a zonally varying basic flow. J. Atmos. Sci, 58, 608-627.
+      #
+      def waf_tn2001(gph, gphbs, nx: nil, ny: nil, ubs: nil, vbs: nil,
+                     zonal: false, merid: false, n2: nil,
+                     derive_M0: false)
 
+        #< prepration >
+        if self==QG
+          f = f0
+          wgt = nil
+        elsif self==QG_sphere
+          f = f_mask0(gph)
+          wgt = cos_phi(gph)
+        else
+          raise("unsupported module to mix-in #{self}")
+        end
+        ret_hash = Hash.new
+
+        #< perturbation stream function >
+        gf = Met::g / f
+        pp = (gph - gphbs)*gf   # psi' (perturbation stream function)
+        if /gpm/i =~ (us=pp.units.to_s)
+          pp.units = us.sub(/gpm/i,'m')
+        end
+        ret_hash[:pp] = pp
+
+        #< derive nx, ny if not given >
+        ul = psibs = nil
+        if zonal
+          nx = 1.0
+          ny = 0.0
+        elsif merid
+          nx = 0.0
+          ny = 1.0
+        elsif nx.nil? && ny.nil?
+          if !ubs || !vbs
+            psibs, gpref = gph2psi_gpref(gphbs)
+            if /gpm/i =~ (us=psibs.units.to_s)
+              psibs.units = us.sub(/gpm/i,'m')
+            end
+            ubs, vbs = psi2ug_vg(psibs)
+            ret_hash[:ubs] = ubs
+            ret_hash[:vbs] = vbs
+            ret_hash[:gpref] = gpref
+          end
+          ul = (ubs**2+vbs**2).sqrt
+          nx = ubs/ul
+          ny = vbs/ul
+        end
+
+        #< derive aspect ratio >
+        unless n2
+          n2 = LogP::gph2n2(gphbs)
+          ret_hash[:n2] = n2
+        end
+        eps = f**2/n2
+
+        #< cross terms --> WAF >
+        px,py = grad_h(pp)
+        pxx,pxy = grad_h(px)
+        pyy = grad_hy(py) unless ny==0.0
+        pz = LogP::pcdata_dz(pp)
+        pxz = LogP::pcdata_dz(px) unless nx==0.0
+        pyz = LogP::pcdata_dz(py) unless ny==0.0
+        ret_hash[:px] = px
+        ret_hash[:py] = py
+
+        wx = nil
+        unless nx==0.0
+          wx = px*px - pp*pxx
+          wy = px*py - pp*pxy
+          wz = ( px*pz - pp*pxz ) * eps
+          nx2 = nx/2.0
+          wx = wx * nx2
+          wy = wy * nx2
+          wz = wz * nx2
+        end
+
+        unless ny==0.0
+          wx_ = px*py - pp*pxy
+          wy_ = py*py - pp*pyy
+          wz_ = (py*pz - pp*pyz) * eps
+          ny2 = ny/2.0
+          wx_ = wx_ * ny2
+          wy_ = wy_ * ny2
+          wz_ = wz_ * ny2
+          if wx
+            wx += wx_
+            wy += wy_
+            wz += wz_
+          else
+            wx = wx_
+            wy = wy_
+            wz = wz_
+          end
+        end
+
+        if wgt
+          wx = wx * wgt
+          wy = wy * wgt
+          wz = wz * wgt
+        end
+        wx.name = "Wx"
+        wy.name = "Wy"
+        wz.name = "Wz"
+        wx.long_name = "WAFx"
+        wy.long_name = "WAFy"
+        wz.long_name = "WAFz"
+
+        #< derive M0 if required >
+        if derive_M0
+          raise("ubs and vbs are needed to derive M0") if !ubs || !vbs
+          ul = (ubs**2+vbs**2).sqrt unless ul
+          unless psibs
+            psibs, gpref = gph2psi_gpref(gphbs)
+            if /gpm/i =~ (us=psibs.units.to_s)
+              psibs.units = us.sub(/gpm/i,'m')
+            end
+            ret_hash[:gpref] = gpref
+          end
+          n2p = Planet::ave_s(n2)
+          if n2p.rank >= 2
+            pdim = Met.find_prs_d(n2p)
+            ds = Array.new
+            (0...n2p.rank).each do |d| 
+              ds.push(d) unless d == pdim
+            end
+            n2p = n2p.mean(*ds)
+          end
+          qp, = psi2q(pp,n2p)
+          qbs, = psi2q(psibs,n2p)
+          qbx, qby = grad_h(qbs)
+          e2 = px**2 + py**2 + pz**2 * eps  # 2*e
+          qbgrad = (qbx**2+qby**2).sqrt.running_mean(0,3).running_mean(1,3)
+          a = qp**2 / qbgrad / 4.0   # A of TN2001
+          e0 = e2 / ul / 4.0         # E of TN2001 when cp=0
+          m0 = a + e0
+          a.name = "A"
+          a.long_name = "A"
+          e0.name = "E0"
+          e0.long_name = "stationary E"
+          m0.name = "M0"
+          m0.long_name = "stationary TN2001 pseudomomentum"
+          ret_hash[:qp] = qp
+          ret_hash[:qbs] = qbs
+          ret_hash[:A] = a
+          ret_hash[:E0] = e0
+          ret_hash[:M0] = m0
+        end
+
+        #< return >
+        [wx, wy, wz, ret_hash]
+      end
    end
 
     ######################################################
@@ -409,6 +619,12 @@ module NumRu
       def grad_h(gphys)
         @@bp.grad_h(gphys)
       end
+      def grad_hy(gphys)
+        @@bp.grad_y(gphys)
+      end
+      def grad_hx(gphys)
+        @@bp.grad_x(gphys)
+      end
 
       # horizontal divergence (Cartesian)
       def div_h(gphys_u, gphys_v)
@@ -442,9 +658,8 @@ module NumRu
 
       # geopotential height -> geostrophic winds
       def gph2ug_vg(gph)
-        gpx, gpy = Planet::grad_s(gph * Met::g)
-        f = f_mask0(gph)
-        [ -gpy/f, gpx/f]
+        psi, gpref = gph2psi_gpref(gph)
+        psi2ug_vg(psi)
       end
 
       # geopotential height -> QG stream function and the reference geopotential
@@ -524,6 +739,12 @@ module NumRu
       def grad_h(gphys)
         Planet::grad_s(gphys)
       end
+      def grad_hx(gphys)
+        Planet::grad_sx(gphys)
+      end
+      def grad_hy(gphys)
+        Planet::grad_sy(gphys)
+      end
 
       # horizontal divergence (spherical)
       def div_h(fx, fy)
