@@ -709,7 +709,7 @@ module NumRu
     def list_dirs_v(path = nil)
       # verbose version
       if path
-	self.dir(path).list_dirs
+	self.dir(path).list_dirs_v
       else
 	if @dir
 	  if /^http:\/\// =~ @abspath
@@ -747,7 +747,7 @@ module NumRu
     def list_data_v(path = nil)
       # verbose version
       if path
-	self.dir(path).list_data
+	self.dir(path).list_data_v
       else
 	if @file
 	  GPhys::IO::var_names(@file).collect do |nm|
@@ -790,7 +790,7 @@ module NumRu
     def list_texts_v(path = nil)
       # verbose version
       if path
-	self.dir(path).list_texts
+	self.dir(path).list_texts_v
       else
 	if @dir
 	  if /^http:\/\// =~ @abspath
@@ -946,7 +946,8 @@ module NumRu
 
     def __can_be_a_gdir(path)
       ftype = __ftype(path)
-      if ftype=="directory" || ftype=="file" && GPhys::IO::file2type(path)
+      if ftype=="directory" || ftype=="file" && 
+          ( GPhys::IO::file2type(path) rescue false )
 	true
       else
 	false
