@@ -630,7 +630,7 @@ module NumRu
 	 end
 
 	 case slicer
-	 when Fixnum
+	 when Integer
 	    pos=@pos[slicer]
 	    cval = ( pos.val.is_a?(Numeric) ? pos.val : pos.val[0] )
 	    units = pos.get_att('units')
@@ -1063,7 +1063,9 @@ module NumRu
 	    @integ_weight = posv.dup
 	    @integ_weight[0] = (posv[1]-posv[0]).abs/2
 	    @integ_weight[-1] = (posv[-1]-posv[-2]).abs/2
-	    @integ_weight[1..-2] = (posv[2..-1] - posv[0..-3]).abs/2
+            if posv.length > 2
+	      @integ_weight[1..-2] = (posv[2..-1] - posv[0..-3]).abs/2
+            end
 	 else
 	    # --- assume that the center values represents the averages ---
 	    bd = @cell_bounds.val
