@@ -16,12 +16,18 @@
   2005/08/23  S Takehiro (created)
   2008/02/14  S Takehiro (copy_global_att can deal with GPhys 
                           which consists of multiple files)
+  2015/09/06  H Kashimura && S Takehiro (fixed for installation by gem)
 
 =end
 
 #------------------------ print help message ------------------------
 def help
-  file = File.open($0)
+  begin 
+    file = File.open(Gem.bin_path("gphys",File.basename($0)))
+  rescue
+    file = File.open($0)
+  end
+#  file = File.open($0)
   after_begin = false
   after_end = false
   while (line = file.gets)
