@@ -33,6 +33,7 @@ module NumRu
     #   This parameter is used only when the data have missing.
     #   Minimum number of grid points needed for averaging.
     #   Must be from 1 to len.
+    # * dont_expand (t/f; default: false): true -> keep where missing as missing
     # 
     # RETURN VALUE
     # * a GPhys
@@ -44,7 +45,8 @@ module NumRu
     # * Regardless the na_type of self, double is used for averaging, so:
     # * This method does not support complex numbers.
     # 
-    def running_mean(dim, len_or_wgt=nil, bc=nil, nminvalid=1)
+    def running_mean(dim, len_or_wgt=nil, bc=nil, nminvalid=1,
+                     dont_expand=false)
 
       #< process arguments >
 
@@ -81,9 +83,19 @@ module NumRu
         raise("This method supports only real or integer data")
       end 
       if vi.is_a?(NArrayMiss)
+        omask = vi.get_mask if dont_expand
         vi, missval = nam2na_missval(vi)
         vo = c_running_mean(vi,dim,wgt,bc,missval,nminvalid)
-        vo = NArrayMiss.to_nam(vo, vo.ne(missval) )
+        mask = vo.ne(missval)
+        if dont_expand
+          if (bc ==  BC_TRIM)
+            fst = (len-1)/2    # if odd len/2, if even len/2-1
+            lst = -(len/2) - 1
+            omask = omask[ *([true]*dim + [fst..lst, false]) ]
+          end
+          mask = mask & omask
+        end
+        vo = NArrayMiss.to_nam(vo, mask)
       else
         vo = c_running_mean(vi,dim,wgt,bc)
       end
@@ -134,7 +146,7 @@ module NumRu
     #
     def running_mean_2D(dim1, dim2, len_or_wgt1, len_or_wgt2,
                         bc1=nil, bc2=nil,
-                        nminvalid=1)
+                        nminvalid=1, dont_expand=false)
 
       dim1 = dim_index(dim1)  # to handle String or negative specification
       dim2 = dim_index(dim2)  # to handle String or negative specification
@@ -157,7 +169,7 @@ module NumRu
       vi = self.val
       if vi.is_a?(NArray)
         # Just for convenience.
-        self.running_mean(dim1, len_or_wgt1, bc1, nminvalid).running_mean(dim2, len_or_wgt2, bc2, nminvalid)
+        self.running_mean(dim1, len_or_wgt1, bc1, nminvalid, dont_expand).running_mean(dim2, len_or_wgt2, bc2, nminvalid, dont_expand)
       end
       
       if (vi.typecode > NArray::DFLOAT)
@@ -166,8 +178,12 @@ module NumRu
       if vi.is_a?(NArrayMiss)
         vi, missval = nam2na_missval(vi)
         vo = c_running_mean_2D(vi, dim1,len_or_wgt1,bc1, dim2,len_or_wgt2,bc2,
-                missval, nminvalid)
-        vo = NArrayMiss.to_nam(vo, vo.ne(missval) )
+                               missval, nminvalid)
+        mask = vo.ne(missval)
+        if dont_expand
+          mask = mask & vi.ne(missval)
+        end
+        vo = NArrayMiss.to_nam(vo, mask)
       else
         raise("This method accepts only data with missing. If not, you should apply running_mean twice for dim1 & dim2, respectively")
       end
@@ -225,6 +241,53 @@ module NumRu
 
     #def convolution(dim, g=nil, bc=BC_SIMPLE)
     #end
+
+   
+    # find the positions or the values of y where the values of z crosses zval along dim (using the linear interpolation)
+    # 
+    # * dim [Integer or String]
+    # * y [nil or VArray of float/sfloat] a coordinate variable or a NArray
+    #   having the same shape as self (but data missig is ignored).
+    #   If nil, dim indices are used.
+    # * nth [Integer] 1,2,3,... for the 1st, 2nd, 3rd,... crossing
+    # * descend [true, false, or nil] true: inspect dim indices from the last
+    # * incr [nil, true, or false] nil: any crsossing is detected; true:
+    #   only increasing crossing is detected; false: only decreasing crossing
+    #   is detected
+    #
+    def val_crossing(dim, zval, y: nil, nth: 1, descend: false, incr: nil)
+      z = self.val
+      if z.is_a?(NArrayMiss)
+        mask = z.get_mask
+        z = z.to_na
+      else
+        mask = nil
+      end
+      dim = dim_index(dim)
+      if y.nil?
+        yv = nil
+      elsif y.is_a?(VArray) || y.is_a?(GPhys)
+        y = y.data if y.is_a?(GPhys)
+        yv = y.val
+        if y.is_a?(NArrayMiss)
+          yv = yv.to_na  # ignore missing
+        end
+      else
+        raise("y must be a VArray or GPhys")
+      end
+      yc,ycmk = GPhys.c_val_crossing(z, mask, dim, zval, yv, nth, descend, incr)
+      yc = NArrayMiss.to_nam(yc,ycmk)
+      if y
+        ycva = VArray.new(yc, y, y.name)
+      else
+        ycva = VArray.new(yc, {"units"=>"", "long_name"=>"index"}, "index")
+      end
+      axes = []
+      (0...rank).each{|d|
+        axes.push(axis(d)) if d != dim
+      }
+      GPhys.new( Grid.new(*axes), ycva )
+    end
   end
 
   class VArray
