@@ -526,6 +526,36 @@ module NumRu
       end
     end
 
+    def fft_filter(dim, iwnmin = nil, iwnmax = nil)
+      dim = dim_index(dim)
+      fc = self.fft(false,dim)
+      mask = val.get_mask
+      len = shape[dim]
+      raise("meaningless unless iwnmin or iwnmax is set") if !iwnmin && !iwnmax
+      if iwnmin
+        iwnmin += len if iwnmin<0
+        raise("invalid iwnmin #{iwnmin}") if iwnmin < 0 || iwnmin > len/2
+      end
+      if iwnmax
+        iwnmax += len if iwnmax<0
+        raise("invalid iwnmax #{iwnmax}") if iwnmax < 0 || iwnmax > len/2
+      end
+      if iwnmax && iwnmin
+        raise("iwnmin>iwnmax") if iwnmin>iwnmax
+      end
+      s = [true]*dim
+      if iwnmin
+        fc[ *(s + [0..iwnmin-1,false]) ] = 0.0  if iwnmin >= 1
+        fc[ *(s + [-iwnmin+1..-1,false]) ] = 0.0 if iwnmin > 1
+      end
+      if iwnmax && iwnmax < len/2
+        fc[ *(s + [iwnmax+1..-iwnmax-1,false]) ] = 0.0
+      end
+      filted = fc.fft(true,dim).to_type(self.data.typecode)
+      filted.replace_val( NArrayMiss.to_nam(filted.val, mask) )
+      filted
+    end
+
     def __predefined_coord_units_conversion(coord)
       case coord.units
       when Units["degree"]
