@@ -25,6 +25,8 @@ module NumRu
 
 	if !files.is_a?(Array)
 	  files = [files]     # put in an Array (if a single File)
+        else
+	  files = files.clone   # will be destroyed by shifting
 	end
 
 	if !loopdims.is_a?(Array)
