@@ -340,6 +340,7 @@ module NumRu
       "cray_32bit_ieee"=>nil,
       "365_day_calendar"=>nil,
       "pdef"=>nil,
+      "edef"=>nil,
     }
 
     case(@mode)
@@ -380,7 +381,7 @@ module NumRu
   end
 
   def ndims
-    4
+    @options["edef"] ? 5 : 4
   end
 
   def nvars
@@ -480,7 +481,7 @@ module NumRu
     end
     @title = "<no title>" if @title==""
     @undef = -999.0 if @undef==nil
-    return <<EOS
+    ctl = <<EOS
 DSET    #{if @dset[0]=="/" then @dset else "^"+@dset end}
 TITLE   #{@title}
 UNDEF   #{@undef}
@@ -489,15 +490,23 @@ XDEF    #{@dimensions[0][:len]} #{@dimensions[0][:flag]} #{@dimensions[0][:spec]
 YDEF    #{@dimensions[1][:len]} #{@dimensions[1][:flag]} #{@dimensions[1][:spec]}
 ZDEF    #{@dimensions[2][:len]} #{@dimensions[2][:flag]} #{@dimensions[2][:spec]}
 TDEF    #{@dimensions[3][:len]} #{@dimensions[3][:flag]} #{@dimensions[3][:spec]}
+EOS
+    if ( @options["edef"] )
+    ctl += <<EOS
+EDEF    #{@dimensions[4][:len]} #{@dimensions[4][:flag]} #{@dimensions[4][:spec]}
+EOS
+    end
+    ctl += <<EOS
 VARS    #{nvars}
 #{@variables.collect{|i| i[:name]+" "+i[:nlev].to_s+" "+i[:option].to_s+
        " "+i[:description]}.join("\n")}
 ENDVARS
 EOS
+    return ctl
   end
 
   def inspect
-    return <<EOS
+    str = <<EOS
 #{self.class}
 file: #{@ctlfile.path}
 DSET #{@dset}
@@ -506,11 +515,19 @@ XDIM #{@dimensions[0].inspect}
 YDIM #{@dimensions[1].inspect}
 ZDIM #{@dimensions[2].inspect}
 TDIM #{@dimensions[3].inspect}
+EOS
+    if ( @options["edef"] )
+      str += <<EOS
+EDEF #{@dimensions[4].inspect}
+EOS
+    end
+    str += <<EOS
 VARS
 #{@variables.collect{|i| "  "+i[:name]+"\t"+i[:nlev].to_s+"\t"+i[:option].to_s+
        "\t"+i[:description]}.join("\n")}
 ENDVARS
 EOS
+    return str
   end
 
   def put(ary)
@@ -530,7 +547,7 @@ EOS
 
   end
 
-  def get(name, z, t, lonlat=nil)
+  def get(name, z, t, lonlat=nil, ens=nil)
 
 # t:    [0,1,2,..] : record number
 # time: [0,1/24,2/24,...] days since 00:00Z01jan2000 : Numeric with units
@@ -593,6 +610,9 @@ EOS
         init_min = target_date.min
         @dset_r.gsub!( /%n2/, sprintf("%02d",init_min) )
       end
+      if ( ens && @dset_r =~ /%e(\d+)/ )
+        @dset_r.gsub!( /%e(\d+)/, sprintf("%0#{$1}d",ens) )
+      end
 
       # find t in the first record in the target file (="init_t")
       # and determine the record number in the target file
@@ -609,10 +629,10 @@ EOS
 
       t_in_target_file = t - init_t
 
-      start_byte = start_byte(name, z, t_in_target_file)
+      start_byte = start_byte(name, z, t_in_target_file, 0)
       @datafile = File.open(@dset_r,"rb")
     else
-      start_byte = start_byte(name, z, t)
+      start_byte = start_byte(name, z, t, ens)
       @datafile = File.open(@dset,"rb")
     end
 
@@ -777,9 +797,8 @@ EOS
         else
           raise "Invalid line: "+line
         end
-      when /^\s*[XYZT]DEF/i
-#        /^\s*([XYZT])DEF\s+(\d+)\s+(\S+)\s+(.*)$/i =~ line
-        /^\s*([XYZT])DEF\s+(\d+)\s+(\S+)+(.*)$/i =~ line
+      when /^\s*[XYZTE]DEF/i
+        /^\s*([XYZTE])DEF\s+(\d+)\s+(\S+)+(.*)$/i =~ line
         if ( (len=$2) && (flag=$3) && (spec=$4))
           dim = {:len=>len.to_i, :flag=>flag, :spec=>spec}
           case $1
@@ -802,8 +821,13 @@ EOS
             dim[:name] = 't'
             dim[:description] = 'time'
             idim=3
+          when /E/i
+            dim[:name] = 'e'
+            dim[:description] = 'ensemble'
+            idim=4
+            @options["edef"] = true
           end
-          if (idim!=3)
+          if (idim<3)
             if (dim[:flag] =~ /LINEAR/i)
               begin
                 dim[:start],dim[:increment] = spec.split.collect!{|i| i.to_f}
@@ -833,7 +857,7 @@ EOS
             else
               raise "invalid or not-yet-supported dimension flag: "+dim[:flag]
             end
-          else
+          elsif idim==3
             # idim = 3 --- time
             if (dim[:flag] =~ /LINEAR/i)
               start,increment= spec.split
@@ -845,6 +869,8 @@ EOS
             else
               raise "invalid dimension flag(only LINEAR is available for time)"
             end
+          else # idim==4 ensemble
+            dim[:levels] = NArray.sint(dim[:len]).indgen
           end
           @dimensions[idim]=dim
         else
@@ -944,6 +970,7 @@ EOS
         sy = sy*PI/180
         sy2 = sy2*PI/180
         xi = NArray.sfloat(imax).indgen(1) - bi
+        bj = jmax - bj if @options['yrev']
         yj = NArray.sfloat(jmax).indgen(1) - bj
         x = xi*dx
         #y = -yj*dy 
@@ -980,7 +1007,6 @@ EOS
         lon = NArray.sfloat(isize.to_i).indgen(1)
         lat = NArray.sfloat(jsize.to_i).indgen(1)
       end
-      p lon, lat
 
       dim = {:len=>isize.to_i, :flag=>"levels"}
       dim[:name] = 'x'
@@ -1012,7 +1038,7 @@ EOS
 
   end
 
-  def start_byte(name, level, time)
+  def start_byte(name, level, time, ens=0)
     # offset to read an xy section of the variable with NAME
     # at LEVEL(counted from 0) at TIME(conted from 0)
     if (map = @map[name] )
@@ -1023,6 +1049,12 @@ EOS
         raise "Time #{time} is not in the data period"
       end
       iblock = map[:start]+level*map[:zstep]+time*map[:tstep]
+      if @options["edef"]
+        if (ens<0 || ens>=@dimensions[4][:len])
+          raise "Ens #{ens} is not in the data"
+        end
+        iblock += ens * map[:tstep] * @dimensions[3][:len]
+      end
       str_byte = map[:offset] + @dimensions[0][:len] * @dimensions[1][:len] * iblock
       if( @options["sequential"] )
         str_byte += iblock*2 + 4
@@ -1243,6 +1275,7 @@ EOS
           @shape = [@dimensions[0][:len],@dimensions[1][:len],
                     @dimensions[2][:len],@dimensions[3][:len]]
           @shape[2] = var[:nlev].to_i if( var[:nlev] )
+          @shape[4] = @dimensions[4][:len] if @rank==5
 
           @attr[:missing_value] = NArray.sfloat(1).fill!(@ctl.undef)
           found = true
@@ -1414,8 +1447,8 @@ EOS
         end
       end
     else
-      h_sta = [0, 0, 0, 0]
-      h_end = [-1, -1, -1, -1]
+      h_sta = [0, 0, 0, 0, 0]
+      h_end = [-1, -1, -1, -1, -1]
       if hash == nil
 #      elsif hash.key?("index")==true
       else
@@ -1439,12 +1472,27 @@ EOS
       str_z = idx_z[h_sta[2]]; end_z = idx_z[h_end[2]]
       str_t = idx_t[h_sta[3]]; end_t = idx_t[h_end[3]]
 
-      na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
-      for t in str_t..end_t
-        for z in str_z..end_z
-          na_xy = @ctl.get(@varname,z,t,[str_x,end_x,str_y,end_y])
-          mask = ( na_xy.ne @ctl.undef )
-          na[true,true,z-str_z,t-str_t] = NArrayMiss.to_nam(na_xy,mask)
+      if @rank == 5 # has ensemble dimension
+        idx_e = NArray.int(@shape[4]).indgen!
+        str_e = idx_e[h_sta[4]]; end_e = idx_e[h_end[4]]
+        na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1,end_e-str_e+1)
+        for e in str_e..end_e
+          for t in str_t..end_t
+            for z in str_z..end_z
+              na_xy = @ctl.get(@varname,z,t,[str_x,end_x,str_y,end_y],e)
+              mask = ( na_xy.ne @ctl.undef )
+              na[true,true,z-str_z,t-str_t,e-str_e] = NArrayMiss.to_nam(na_xy,mask)
+            end
+          end
+        end
+      else
+        na = NArrayMiss.new(vartype,end_x-str_x+1,end_y-str_y+1,end_z-str_z+1,end_t-str_t+1)
+        for t in str_t..end_t
+          for z in str_z..end_z
+            na_xy = @ctl.get(@varname,z,t,[str_x,end_x,str_y,end_y])
+            mask = ( na_xy.ne @ctl.undef )
+            na[true,true,z-str_z,t-str_t] = NArrayMiss.to_nam(na_xy,mask)
+          end
         end
       end
     end
@@ -1479,7 +1527,7 @@ EOS
      stride = Array.new
      set_stride = false
      a.each{|i|
-       if(i.is_a?(Fixnum))
+       if(i.is_a?(Integer))
          first.push(i)
          last.push(i)
          stride.push(1)
@@ -1511,7 +1559,7 @@ EOS
            if n==0 then
              k = at
              if at > 0
-               a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+               a[0..at-1].each{|x| if x.is_a?(Integer) then k -= 1 end}
              end
              shape_tmp = na_tmp.shape
              shape_tmp[k] = i.length
@@ -1523,7 +1571,7 @@ EOS
          end
          return na
        else
-         raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
+         raise TypeError, "argument must be Integer, Range, Hash, TrueClass, Array, or NArray"
        end
      }
 
@@ -1534,7 +1582,7 @@ EOS
      end
      shape = na.shape
      (a.length-1).downto(0){ |i|
-         shape.delete_at(i) if a[i].is_a?(Fixnum)
+         shape.delete_at(i) if a[i].is_a?(Integer)
       }
       na.reshape!( *shape )
      na
@@ -1548,7 +1596,7 @@ EOS
      stride = Array.new
      set_stride = false
      a.each{|i|
-       if(i.is_a?(Fixnum))
+       if(i.is_a?(Integer))
          first.push(i)
          last.push(i)
          stride.push(1)
@@ -1575,13 +1623,13 @@ EOS
          at = a.index(i)
          i = NArray.to_na(i) if i.is_a?(Array)
          val = NArray.to_na(val) if val.is_a?(Array)
-         rank_of_subset = a.dup.delete_if{|v| v.is_a?(Fixnum)}.length
+         rank_of_subset = a.dup.delete_if{|v| v.is_a?(Integer)}.length
          if val.rank != rank_of_subset
            raise "rank of the rhs (#{val.rank}) is not equal to the rank "+
                  "of the subset specified by #{a.inspect} (#{rank_of_subset})"
          end
          k = at
-         a[0..at-1].each{|x| if x.is_a?(Fixnum) then k -= 1 end}
+         a[0..at-1].each{|x| if x.is_a?(Integer) then k -= 1 end}
          if i.length != val.shape[k]
            raise "length of the #{k+1}-th dim of rhs is incorrect "+
                  "(#{i.length} for #{val.shape[k]})"
@@ -1598,7 +1646,7 @@ EOS
          end
          return self
        else
-         raise TypeError, "argument must be Fixnum, Range, Hash, TrueClass, Array, or NArray"
+         raise TypeError, "argument must be Integer, Range, Hash, TrueClass, Array, or NArray"
        end
      }
 
