@@ -1229,7 +1229,7 @@ class GPhys
           nlon,nlat = shape
           (nlon*nlat)==nlen+nmiss || raise("length is not collect")
           if index
-            index = index.collect{|el| Fixnum===el ? el..el : el}
+            index = index.collect{|el| Integer===el ? el..el : el}
             il = index[0]
             i0 = il.first
             i1 = il.last
@@ -1980,7 +1980,7 @@ even when the time is unique (i.e., length of the time coordinate is 1).
                 (0..dlen-1).each{|i| imap[n-2][i] = i}
               end
               next
-            when Fixnum
+            when Integer
               sha[n] = 1
               imap[n-2][ind] = 0
               dimdel.push(n)
