@@ -706,8 +706,8 @@ NAMES_UNITS = {
 #150	COVMZ	m2/s2		Covariance between meridional and zonal components of the wind. Defined as [uv]-[u][v], where "[]" indicates the mean over the indicated time span.		
 #151	COVTZ1	K*m/s		Covariance between temperature and zonal component of the wind. Defined as [uT]-[u][T], where "[]" indicates the mean over the indicated time span.		
 #152	COVTM	K*m/s		Covariance between temperature and meridional component of the wind. Defined as [vT]-[v][T], where "[]" indicates the mean over the indicated time span.		
-  PARAM_CLWMR => ["Cloud water", "CLWMR", "Kg/kg", nil],
-  PARAM_O3MR => ["Ozone mixing ratio", "O3MR", "Kg/kg", nil],
+  PARAM_CLWMR => ["Cloud water", "CLWMR", "kg/kg", nil],
+  PARAM_O3MR => ["Ozone mixing ratio", "O3MR", "kg/kg", nil],
 #155	GFLUX	W/m2		Ground Heat Flux		
   PARAM_CIN => ["Convective inhibition", "CIN", "J/kg", nil],
   PARAM_CAPE => ["Convective Available Potential Energy", "CAPE", "J/kg", nil],		
@@ -941,7 +941,7 @@ PARAM_CWORK => ["Cloud work function", "CWORK", "J/kg", nil],
 PARAM_FGLU => ["U sfc momentum flux by long gravity wave", "FGLU", "N/m^2", nil],
 PARAM_FGLV => ["V sfc momentum flux by long gravity wave", "FGLV", "N/m^2", nil],
 PARAM_ADVUA => ["Advective zonal acceleration", "ADVUA", "m/s/day", nil],
-PARAM_VWV => ["Column total of meridional water vapor flux", "VWV", "Kg/m/s", nil],
+PARAM_VWV => ["Column total of meridional water vapor flux", "VWV", "kg/m/s", nil],
 PARAM_FGSV => ["V sfc momentum flux by short gravity wave", "FGSV", "N/m^2", nil],
 PARAM_GFLX => ["Ground heat flux", "GFLX", "W/m^2", nil],
 PARAM_UWV => ["Column total of zonal water vapor flux", "UWV", "kg/m/s", nil],
