@@ -186,16 +186,16 @@ A class to handle discretized grids of physical quantities.
 #    RETURN VALUE
 #    * self
 #
-#---insert_axis(dim_or_dimname, axis)
-#    Inserts an axis. (Returns a new object)
-#
-#    ARGUMENTS
-#    * dim_or_dimname (String or Integer) to specify an axis.
-#    * axis (Axis)
-#
-#    RETURN VALUE
-#    * a Grid
-#
+---insert_axis(dim, axis)
+    Inserts an axis. (non destructive; returns a new object)
+
+    ARGUMENTS
+    * dim (Integer) to specify the position to insert
+    * axis (Axis)
+
+    RETURN VALUE
+    * a Grid
+
 #---insert_axis!(dim_or_dimname, axis)
 #    Inserts an axis. (overwrites self)
 #
@@ -225,11 +225,12 @@ module NumRu
       def initialize( *axes )
 	 @axes = Array.new
 	 axes.each{|ag|
-	    if ag.is_a?(Axis)
-	       @axes.push(ag)
-	    else
-	       raise ArgumentError, "each argument must be an Axis"
-	    end
+            ### commented out for the duck typing:
+            #if ag.is_a?(Axis)
+            @axes.push(ag)
+            #else
+            #   raise ArgumentError, "each argument must be an Axis"
+	    #end
 	 }
 	 @lost_axes = Array.new   # Array of String
 	 @rank = @axes.length
@@ -409,7 +410,18 @@ module NumRu
 	newgrid = self.class.new( *newaxes )
 	newgrid.set_lost_axes( @lost_axes.dup )
         if @assoc_coords
-          newgrid.assoc_coords=@assoc_coords.subset_having_axnames(newgrid.axnames)
+          if deleted_by && VArray::NArray_type3_methods.include?(deleted_by)
+            assoc_coords = Array.new
+            assoccoordnames.each do |aname|
+              g = assoc_coord_gphys(aname)
+              args = g.axnames & at.collect{|i| @axnames[i]}
+              g = g.mean(*args) unless args.empty?
+              assoc_coords.push g if GPhys === g
+            end
+            newgrid.set_assoc_coords assoc_coords
+          else
+            newgrid.assoc_coords = @assoc_coords.subset_having_axnames(newgrid.axnames)
+          end
         end
 
 	if !deleted_by 
@@ -466,6 +478,26 @@ module NumRu
       end
       alias shape_current shape
 
+      # similar to [] but only for cut
+      def _new_grid_for_cut(slicer, axes)
+        axs = Array.new
+        lost = self.lost_axes  #Array.new
+        for i in 0...rank
+          if axes[i].is_a?(String)
+            lost.push( axes[i] )
+          else
+            axs.push( axes[i] )
+          end
+        end
+        grid = self.class.new( *axs )
+        grid.set_lost_axes( lost ) if lost.length != 0
+        if @assoc_coords
+          grid.assoc_coords = @assoc_coords[*slicer]
+        end
+        grid
+      end
+      private :_new_grid_for_cut
+
       def [] (*slicer)
 	 if slicer.length == 0
 	    # make a clone
@@ -481,15 +513,14 @@ module NumRu
 	    lost = self.lost_axes  #Array.new
 	    for i in 0...rank
 	       ax = @axes[i][slicer[i]]
-	       if ax.is_a?(Axis)      # else its rank became zero (lost)
-		  axes.push( ax )
+	       if ax.is_a?(String)
+                 lost.push( ax )
                else
-		  lost.push( ax )
+                 axes.push( ax )
 	       end
 	    end
 	    grid = self.class.new( *axes )
 	    grid.set_lost_axes( lost ) if lost.length != 0
-	    grid
 	 end
          if @assoc_coords
            grid.assoc_coords = @assoc_coords[*slicer]
@@ -550,17 +581,18 @@ module NumRu
 	end
 
 	slicer = Array.new
+	axes = Array.new
 
 	for dim in 0...rank
 	  ax = @axes[dim]
 	  if conserve_rank
-	    dummy, slicer[dim] = ax.cut_rank_conserving(args[dim])
+	    axes[dim], slicer[dim] = ax.cut_rank_conserving(args[dim])
 	  else
-	    dummy, slicer[dim] = ax.cut(args[dim])
+	    axes[dim], slicer[dim] = ax.cut(args[dim])
 	  end
 	end
 
-	[ self[*slicer], slicer ]
+	[ _new_grid_for_cut(slicer,axes), slicer ]
       end
       private :_cut_
 
@@ -599,17 +631,6 @@ module NumRu
 #	 self
 #      end
 
-#      def insert_axis(dim_or_dimname, axis)
-#	 dim = dim_index(dim_or_dimname)
-#	 axes = @axes.dup
-#	 if axis.is_a?(Axis)
-#	    axes[dim+1,0] = axis    # axes.insert(dim, axis) if ruby 1.7
-#	 else
-#	    # do nothing
-#	 end
-#	 self.class.new( *axes )
-#      end
-
 #      def insert_axis!(dim_or_dimname, axis)
 #	 dim = dim_index(dim_or_dimname)
 #	 if axis == nil
@@ -622,6 +643,20 @@ module NumRu
 #	 self
 #      end
 
+      def insert_axis(dim, axis)
+        axes = @axes.dup
+        ### commented out for the duck typing:
+        #if axis.is_a?(Axis)
+          axes.insert(dim, axis)
+        #else
+        #  raise ArgumentError, "2nd arg must be an Axis"
+        #end
+        grid = self.class.new( *axes )
+	grid.set_lost_axes( lost_axes )
+        grid.assoc_coords = @assoc_coords.dup if @assoc_coords
+	grid
+      end
+
       def transpose( *dims )
 	if dims.sort != NArray.int(rank).indgen!.to_a
 	  raise ArgumentError, 
@@ -719,5 +754,11 @@ if $0 == __FILE__
    p grid.delete_axes([0,1], 'mean').lost_axes
 
    p grid, grid.transpose(1,0)
+
+   puts "Test insert_axis..."
+   vz = VArray.new( NArray.float(3).indgen! + 10 ).rename("z")
+   zax = Axis.new().set_pos(vz)
+   grid3 = grid.insert_axis(-1,zax)
+   p grid3
 end
 
