@@ -210,6 +210,85 @@ module NumRu
       interpolate(*coords)
     end
 
+    # Regrid for equally spaced grid (or index based regrid)
+    # 
+    # ARGUMENTS
+    # * axes [Array of Axis (or 1D VArray)] : if 1D VArray, an Axis is created
+    # * poses [Array of multi-D VArray (or GPhys)] : specifies where to
+    #   regrid. lengths of poses and axes must agree. The shape of each of pos
+    #   must be equal to the shape suggested by axes (i.e., corrected lengths
+    #   of axes). poses must have non-overwrapping names.
+    #   Each pos (an element of poses) must have a name equal to
+    #   a main coordinate varible (or a coordinate variable plus "_index").
+    #   E.g., if self has axes named ["x", "y", "t"], the name of a pos
+    #   must be one of "x", "y", "t", "x_index", "y_index", "t_index".
+    #   No two pos must share the coordinate name (w or wo _index).
+    def regrid2( axes, poses )
+      if axes.length != poses.length
+        raise(ArgumentError, "lengths of axes and poses do not agree")
+      end
+      axes = axes.map{|x| x.is_a?(VArray) ? Axis.new.set_pos(x) : x }
+      newshape = axes.map{|x| x.length}
+      dims = Array.new
+      cyclics = Array.new
+      axnms = axnames
+      eps = 1e-4
+      ndims = poses.length
+      scidxs = poses.map do |pos|
+        if pos.shape != newshape
+          raise(ArgumentError, "lengths of axes and poses do not agree")
+        end
+        nm = pos.name
+        index_type = false  # initialization
+        d = axnms.index(nm)
+        if !d
+          d = axnms.index(nm.sub(/_index$/,''))
+          raise("pos #{nm} does not match any of the axis names") if !d
+          index_type = true
+        end
+        dims.push(d)
+        cyclics.push( axis(d).cyclic_extendible? )
+        v = pos.val
+        if v.is_a?(NArrayMiss)
+          if v.get_mask.count_false == 0
+            v = v.to_na
+          else
+            raise ArgumentError, "pos cannot have data missing"
+          end
+        end
+        if index_type
+          v
+        else
+          cval = coord(d).val
+          c0 = cval[0]
+          dc = (cval[-1] - cval[0])/(shape[d]-1.0)
+          if (cval[1]-cval[0] - dc).abs > eps*dc
+            raise("coordinate #{nm} is not equally spaced." +
+                  " Use index based spacification if possible.")
+          end
+          (v - c0)/dc
+        end
+      end
+      na = val
+      if na.is_a?(NArrayMiss)
+        mask = na.get_mask
+        na = na.to_na
+      else
+        mask = NArray.byte( *na.shape )
+        mask = mask.fill!(1)
+      end
+      
+      rval, rmask = c_regrid2_w_idx(na, mask, dims, scidxs, cyclics)
+
+      newdata = VArray.new( NArrayMiss.to_nam(rval, rmask), @data, name )
+      newaxes = (0...rank).map{|d| axis(d)}
+      (0...dims.length).each do |d|
+        newaxes[ dims[d] ] = axes[d]
+      end
+      newgrid = Grid.new( *newaxes )
+      GPhys.new( newgrid, newdata )
+    end
+    
     # Reverse the main data (i.e., the dependent variable) and one of the
     # coordinates (an independent variable) through interpolation.
     # 
@@ -424,6 +503,50 @@ module NumRu
       ret
     end
 
+    # interpolate data missing alon dim's dimention
+    #
+    # ARGUMENTS:
+    # * dim [Integer or String] specifies the dimension
+    #
+    # * force_cyclic [nil, true, or Numeric] : if nil (default),
+    #   cyclic interpolation is made only if the cyclic_extendible?
+    #   applied to the dimension returns true (that is, the axis
+    #   is cyclic and only one grid-point extension is needed).
+    #   By setting true, cyclic extension is made as long as modulo
+    #   is defined, irrespective of the remaining distances. By
+    #   setting a Numeric, cyclic extension is forced by using its
+    #   value as the modulo.
+    def interpolate_missing(dim, force_cyclic: nil)
+      dim = dim_index(dim)
+      ax = axis(dim)
+      modulo = nil
+      case force_cyclic
+      when nil
+        cyclic = ax.cyclic_extendible?
+        modulo = ax.modulo if cyclic
+      when true
+        modulo = ax.modulo
+        if modulo
+          cyclic = true
+          modulo = modulo[0]
+        else
+          cyclic = false
+        end
+      when Numeric
+        cyclic = true
+        modulo = force_cyclic
+      else
+        raise(ArgumentError, "force_cyclic must be nil, true, or Numeric")
+      end
+      v = self.val
+      na = v.to_na
+      mask = v.get_mask
+      x = ax.pos.val
+      ve, mask = c_interpo_missing(na,mask,dim,x,cyclic,modulo)
+      dat = VArray.new( NArrayMiss.to_nam(ve,mask), self.data, self.name )
+      GPhys.new(grid, dat)
+    end
+    
     private
 
     def _interpo_find_position(crdmap)
@@ -987,7 +1110,36 @@ if $0 == __FILE__
   GGraph.tone gpcut,true,"int"=>0.15,"min"=>-1.05,"max"=>1.05
   GGraph.color_bar
   
+  GGraph::tone gdm,true,"color_bar"=>true, "title"=>"orig with missing"
+  p gdm.coord(1).att_names, gdm.coord(1).val.to_a
+  gde = gdm.interpolate_missing(1)
+  GGraph::tone gde,true,"color_bar"=>true, "title"=>"missing interpolated (1, non-cyclic)"
+  gde = gdm.interpolate_missing(1, force_cyclic: 2*Math::PI)
+  GGraph::tone gde,true,"color_bar"=>true, "title"=>"missing interpolated (1, cyclic)"
+  gde = gdm.interpolate_missing(0, force_cyclic: 2*Math::PI)
+  GGraph::tone gde,true,"color_bar"=>true, "title"=>"missing interpolated (0, cyclic)"
 
+  nth = 40
+  nr = 40
+  theta = NArray.float(nth).indgen! * (Math::PI/2/(nth-1))
+  #p "@@@@",gde.coord(0).val, gde.coord(1).val
+  r = NArray.float(nr).indgen! * (8.0/nr)
+  thax = Axis.new.set_pos( VArray.new(theta,
+                 {"long_name"=>"azimuth","units"=>"radian"}, "theta") )
+  rax = Axis.new.set_pos( VArray.new(r,
+                 {"long_name"=>"r","units"=>"m"}, "r") )
+  x = r.newdim(-1) * NMath.cos( theta.newdim(0) ) + gde.coord(0).val[0]
+  y = r.newdim(-1) * NMath.sin( theta.newdim(0) ) + gde.coord(1).val[0]
+  vx = VArray.new(x, {"long_name"=>"x","units"=>"m"}, "x")
+  vy = VArray.new(y, {"long_name"=>"y","units"=>"m"}, "y")
+
+  GGraph::tone gdd[false,1],true,"color_bar"=>true
+  gdr = gdd.regrid2( [rax, thax], [vx, vy] )
+  GGraph::tone gdr[false,1],true,"color_bar"=>true, "keep"=>true, #"tonc"=>true,
+               "title"=>"polar projected"
+  #GGraph::contour gdr,false
+  p "%%%", gdd, gdr
+  
   #< finish >
   DCL.grcls
 end
