@@ -149,7 +149,9 @@ module NumRu
       # p @bound_idx, @varrays
     end
 
-    undef initialize_mapping
+    [:initialize_mapping, :mapping, :varray, :ary, :reshape].each do |method|
+      undef_method(method) if method_defined?(method)
+    end
 
     def set_att(name,val)
       @varrays.each{|va|
@@ -158,8 +160,6 @@ module NumRu
       self
     end
 
-    undef :mapping, :varray, :ary
-
     def inspect
       "<#{self.class.to_s} shape=#{shape.inspect} #_of_tiles=#{@bound_idx.collect{|b| b.length-1}.inspect}>"
       # "bounds=#{@bound_idx.collect{|b| b[1..-2]}.inspect}"
@@ -203,6 +203,9 @@ module NumRu
 	    raise TypeError, "Unexpected type #{v.class}"
 	  end
 	end
+        if val.is_a?(NArray) && v.is_a?(NArrayMiss)
+          val = NArrayMiss.to_nam(val)
+        end
 	val[*vidx] = v
       }
       val
@@ -286,7 +289,7 @@ module NumRu
     #def reshape!( *shape )
     #  raise "cannot reshape a #{class}. Use copy first to make it a VArray with NArray"
     #end
-    undef reshape!
+    #undef reshape!
 
     ## Not needed, so far:
     #def convention
