@@ -17,10 +17,6 @@ p gp.grid.axis(0).pos.name
 gpmean = gp.mean(0)
 p gpmean.class, gpmean.name, gpmean.rank
 
-class NArray
-  def self._load(o) to_na(*Marshal::load(o)).ntoh end
-end
-
 p ( gpmean_copy = gpmean.copy )
 DCL.gropn(1)
 DCL.sgpset('lfull',true)
@@ -30,5 +26,3 @@ GGraph.set_fig('viewport'=>[0.25,0.7,0.15,0.6], 'itr'=>2 )
 GGraph.contour( gpmean_copy )
 DCL.grcls
 
-print "\nThe following will be a DRbObject rather than GPhys since the size is too large\n(see druby_serv2.rb):\n"
-p ( gpcopy = gp.copy )
